#include "AdaptInfo.hpp"

// std c++ headers
#include <iostream>
#include <string>

#include <amdis/Initfile.hpp>

namespace AMDiS
{

  AdaptInfo::ScalContent::ScalContent(std::string prefix)
  {
    Parameters::get(prefix + "->tolerance", spaceTolerance);
    Parameters::get(prefix + "->time tolerance", timeTolerance);
    Parameters::get(prefix + "->time relative tolerance", timeRelativeTolerance);
    Parameters::get(prefix + "->coarsen allowed", coarsenAllowed);
    Parameters::get(prefix + "->refinement allowed", refinementAllowed);
    Parameters::get(prefix + "->sum factor", fac_sum);
    Parameters::get(prefix + "->max factor", fac_max);

    if (timeTolerance == 0.0 && timeRelativeTolerance == 0.0)
      timeTolerance = 1.0;
    timeErrLow = timeTolerance * 0.3;
  }


  AdaptInfo::AdaptInfo(std::string name)
    : name(name)
  {
    // init();
    Parameters::get(name + "->start time", startTime);
    time = startTime;

    Parameters::get(name + "->timestep", timestep);
    Parameters::get(name + "->end time", endTime);
    Parameters::get(name + "->max iteration", maxSpaceIteration);
    Parameters::get(name + "->max timestep iteration", maxTimestepIteration);
    Parameters::get(name + "->max time iteration", maxTimeIteration);
    Parameters::get(name + "->min timestep", minTimestep);
    Parameters::get(name + "->max timestep", maxTimestep);
    Parameters::get(name + "->number of timesteps", nTimesteps);
    Parameters::get(name + "->time tolerance", globalTimeTolerance);
  }


  void AdaptInfo::printTimeErrorLowInfo() const
  {
    for (auto const& scalContent : scalContents)
    {
      auto i = scalContent.first;
      std::cout << "    Time error estimate     ["<<i<<"] = "
                << getTimeEstCombined(i) << "\n"
                << "    Time error estimate sum ["<<i<<"] = "
                << scalContent.second.est_t_sum << "\n"
                << "    Time error estimate max ["<<i<<"] = "
                << scalContent.second.est_t_max << "\n"
                << "    Time error low bound    ["<<i<<"] = "
                << scalContent.second.timeErrLow << "\n"
                << "    Time error high bound   ["<<i<<"] = "
                << scalContent.second.timeTolerance << "\n";
    }
  }

  void AdaptInfo::reset()
  {
    spaceIteration = -1;
    timestepIteration = 0;
    timeIteration = 0;
    time = 0.0;
    timestep = 0.0;
    timestepNumber = 0;
    solverIterations = 0;
    solverResidual = 0.0;

    Parameters::get(name + "->timestep", timestep);
    lastProcessedTimestep=timestep;
  }

} // end namespace AMDiS
