#pragma once

#include <type_traits>

#include <dune/common/fvector.hh>

#include <amdis/common/IndexSeq.hpp>
#include <amdis/common/MultiTypeVector.hpp>

namespace AMDiS
{
  namespace Impl
  {
    template <class Node, bool isLeaf, bool isPower, bool isComposite>
    struct RangeTypeImpl
    {
      static_assert( isLeaf || isPower || isComposite, "Unknown node-type for range definition" );
    };
  }

  template <class Node>
  using RangeType_t =
    typename Impl::RangeTypeImpl<Node, Node::isLeaf, Node::isPower, Node::isComposite>::type;


  namespace Impl
  {
    // Leaf node
    template <class Node>
    struct RangeTypeImpl<Node, true, false, false>
    {
      using LocalBasis = typename Node::FiniteElement::Traits::LocalBasisType;
      using type = typename LocalBasis::Traits::RangeType;
    };

    // Power node
    template <class Node>
    struct RangeTypeImpl<Node, false, true, false>
    {
      using ChildNode = typename Node::template Child<0>::type;

      template <bool childIsLeaf, class ChildRangeType>
      struct FlatType {
        using type = Dune::FieldVector<ChildRangeType, int(Node::CHILDREN)>;
      };

      template <class T>
      struct FlatType<true, Dune::FieldVector<T,1>> {
        using type = Dune::FieldVector<T, int(Node::CHILDREN)>;
      };

      using type = typename FlatType<ChildNode::isLeaf, RangeType_t<ChildNode>>::type;
    };

    // Composite node
    template <class Node>
    struct RangeTypeImpl<Node, false, false, true>
    {
      template <class Idx> struct RangeTypeGenerator;
      template <std::size_t... I>
      struct RangeTypeGenerator<Indices<I...>>
      {
        template <std::size_t J>
        using ChildNode = typename Node::template Child<J>::type;
        using type = MultiTypeVector<RangeType_t<ChildNode<I>>...>;
      };

      using type = typename RangeTypeGenerator<MakeSeq_t<Node::CHILDREN>>::type;
    };

  } // end namespace Impl

} // end namespace AMDiS
